/**
 * OW2 Util
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: ScriptConditionEvaluator.java 6240 2012-04-18 11:03:10Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.deployer.impl.condition;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptException;

import org.ow2.util.plan.deployer.api.condition.EvaluationException;
import org.ow2.util.plan.deployer.api.condition.IConditionEvaluator;

/**
 * A {@code ScriptConditionEvaluator} is ...
 *
 * @author Guillaume Sauthier
 */
public class ScriptConditionEvaluator implements IConditionEvaluator {

    private ScriptEngineFactory scriptEngineFactory;

    public void setScriptEngineFactory(ScriptEngineFactory scriptEngineFactory) {
        this.scriptEngineFactory = scriptEngineFactory;
    }

    public boolean evaluate(String condition) throws EvaluationException {

        // Create a new engine for this evaluation
        ScriptEngine engine = scriptEngineFactory.getScriptEngine();

        try {

            // Determine if the result should be interpreted as true or false
            return isTrue(engine.eval(condition));

        } catch (ScriptException e) {
            throw new EvaluationException("Error during script evaluation", e);
        }
    }

    private boolean isTrue(Object o) {
        if (o == null) {
            return false;
        }

        if (o instanceof Boolean) {
            return (Boolean) o;
        }

        // If the value is not null, this is sufficient
        return true;
    }
}
