/**
 * OW2 Util
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: RhinoScriptEngineFactory.java 6240 2012-04-18 11:03:10Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.deployer.impl.condition.script;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineFactory;
import javax.script.ScriptException;

/**
 * A {@code RhinoScriptEngineFactory} is ...
 *
 * @author Guillaume Sauthier
 */
public class RhinoScriptEngineFactory extends ForwardingScriptEngineFactory {

    private static final String ENVIRONMENT_JS = "environment.js";
    private ScriptEngineFactory scriptEngineFactory;

    public void setScriptEngineFactory(ScriptEngineFactory scriptEngineFactory) {
        if (!isJavaScriptEngine(scriptEngineFactory)) {
            throw new RuntimeException("Expecting a JavaScript engine");
        }
        this.scriptEngineFactory = scriptEngineFactory;
    }

    private boolean isJavaScriptEngine(ScriptEngineFactory scriptEngineFactory) {
        for (String mimeType : scriptEngineFactory.getMimeTypes()) {
            if ("application/javascript".equals(mimeType)) {
                return true;
            }
        }
        return false;
    }

    @Override
    protected ScriptEngineFactory delegate() {
        return scriptEngineFactory;
    }

    @Override
    public ScriptEngine getScriptEngine() {
        ScriptEngine engine = super.getScriptEngine();

        // Fill the engine with pre-defined functions
        evaluateResource(engine, "SystemPropertyInfo.js");
        evaluateResource(engine, ENVIRONMENT_JS);

        return engine;
    }

    private void evaluateResource(final ScriptEngine engine, final String resourceName) {
        evaluateResource(engine, RhinoScriptEngineFactory.class, resourceName);
    }

    protected void evaluateResource(final ScriptEngine engine,
                                    final Class<?> site,
                                    final String resourceName) {
        Reader reader = null;
        try {
            reader = getResourceReader(site, resourceName);
            engine.eval(reader);
        } catch (ScriptException e) {
            throw new RuntimeException("Cannot execute script " + resourceName, e);
        } catch (IOException e) {
            throw new RuntimeException("Cannot load script " + resourceName, e);
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (IOException e) {
                    // Ignored
                }
            }
        }
    }

    private Reader getResourceReader(final Class<?> site, final String name) throws IOException {
        InputStream is = site.getResourceAsStream(name);
        if (is == null) {
            throw new IOException("Cannot load resource '" + name + "' from " + site);
        }
        return new InputStreamReader(is);
    }


}
